/**
 *	File	: BStringListTest.cpp
 *	Author	: Dariusz "DKnoto" Knociński
 *	Project	: Haiku OS / Terminal
 *	Site	: ul.Gen.Zajączka 21/1, 01-505 Warszawa 
 *	Date	: 2012/12/05 23:31
 *	Notes	: Test properties of the BStringList class.
 */

#include <stdio.h>
#include <posix/strings.h>

#include <os/support/String.h>
#include <os/support/StringList.h>

static int CompareStrAsDecimalValue(const void *ptr1, const void *ptr2)
{
	const char *pStr1 = *(const char **)ptr1;
	const char *pStr2 = *(const char **)ptr2;
	const int len1 = strlen(pStr1);
	const int len2 = strlen(pStr2);
	
	int retval;
	
	if (len1 == len2)
		retval = strcmp(pStr1, pStr2);
	else
	if (len1 < len2)
		retval = -10;
	else
		retval = 10;
	
	printf( "CompareStrAsDecimalValue( \"%s\", \"%s\" ) == %d\n", pStr1, pStr2,
		retval);

	return retval;
}

int main(void)
{
	printf("BStringList test:\n");

	printf("  create default list...\n");
	BStringList *pList = new BStringList();
	printf("  p == %p\n", (void *)pList);

	pList->Add(BString("8"));
	pList->Add("10");
	pList->Add(BString("12"));
	pList->Add("14");
	pList->Add(BString("16"));
	pList->Add("18");

	printf("  pList->CountStrings() == %ld\n", pList->CountStrings());
	printf("  pList->First() == \"%s\"\n", pList->First().String());
	printf("  pList->Last() == \"%s\"\n", pList->Last().String());
	

	const char *strTrue = "True";
	const char *strFalse = "False";
	
	const char *testItem1 = "7";
	printf("  pList->HasString(\"%s\") == \"%s\"\n", testItem1, pList->HasString(testItem1) ? strTrue : strFalse);
	printf("  pList->IndexOf(\"%s\") == %ld\n", testItem1, pList->IndexOf(testItem1));
	printf("  pList->Add(\"%s\") == \"%s\"\n", testItem1, pList->Add(testItem1) ? strTrue : strFalse);
	
	const char *testItem2 = "13";
	printf("  pList->HasString(\"%s\") == \"%s\"\n", testItem2, pList->HasString(testItem2) ? strTrue : strFalse);
	printf("  pList->IndexOf(\"%s\") == %ld\n", testItem2, pList->IndexOf(testItem2));
	printf("  pList->Add(\"%s\") == \"%s\"\n", testItem2, pList->Add(testItem2) ? strTrue : strFalse);
	
	const char *testItem3 = "20";
	printf("  pList->HasString(\"%s\") == \"%s\"\n", testItem3, pList->HasString(testItem3) ? strTrue : strFalse);
	printf("  pList->IndexOf(\"%s\") == %ld\n", testItem3, pList->IndexOf(testItem3));
	printf("  pList->Add(\"%s\") == \"%s\"\n", testItem3, pList->Add(testItem3) ? strTrue : strFalse);

	printf("  All items == { ");
	for (int i=0; i < pList->CountStrings()-1; i++)
	{
		printf( "\"%s\", ", pList->StringAt(i).String());
	}
	printf( "\"%s\" }\n", pList->StringAt(pList->CountStrings()-1).String());

	printf( "  started sorting:\n" );
	pList->Sort(); // 2012/12/06 00:06 - na tej metodzie program się wypierdala!
	
	printf("  All items after sorting == { ");
	for (int i=0; i < pList->CountStrings()-1; i++)
	{
		printf( "\"%s\", ", pList->StringAt(i).String());
	}
	printf( "\"%s\" }\n", pList->StringAt(pList->CountStrings()-1).String());

	printf("  pList->CountStrings() == %ld\n", pList->CountStrings());
	printf("  pList->First() == \"%s\"\n", pList->First().String());
	printf("  pList->Last() == \"%s\"\n", pList->Last().String());
	
	printf( "  started decimal value sorting:\n" );
	pList->Sort(CompareStrAsDecimalValue); // 2012/12/06 00:06 - na tej metodzie program się wypierdala!
	
	printf("  All items after sorting == { ");
	for (int i=0; i < pList->CountStrings()-1; i++)
	{
		printf( "\"%s\", ", pList->StringAt(i).String());
	}
	printf( "\"%s\" }\n", pList->StringAt(pList->CountStrings()-1).String());

	printf("  pList->CountStrings() == %ld\n", pList->CountStrings());
	printf("  pList->First() == \"%s\"\n", pList->First().String());
	printf("  pList->Last() == \"%s\"\n", pList->Last().String());
	
	delete pList;
	pList = NULL;
	
	return 0;
}

/**
 *	EOF
 */
